@@ -38,21 +38,26 @@
  *
  * ***** END LICENSE BLOCK *****
  *
- * $Id: MozDownload.cpp,v 1.11 2003/12/21 23:48:39 marco Exp $
+ * $Id: MozDownload.cpp,v 1.19 2004/01/24 09:56:43 marco Exp $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "MozDownload.h"
 #include "mozilla-download.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include <libgnomevfs/gnome-vfs-utils.h>
+#include <glib/gi18n.h>
 
 #include "nsIExternalHelperAppService.h"
 #include "nsDirectoryServiceDefs.h"
 #include "nsDirectoryServiceUtils.h"
 #include "nsIRequest.h"
+#include "nsIMIMEInfo.h"
 #include "netCore.h"
-#include "nsIObserver.h"
 
 const char* const persistContractID = "@mozilla.org/embedding/browser/nsWebBrowserPersist;1";
 
@@ -85,6 +90,7 @@ MozDownload::Init(nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisp
 		   nsIMIMEInfo *aMIMEInfo, PRInt64 startTime, nsIWebBrowserPersist *aPersist)
 {
 	PRBool addToView = PR_TRUE;
+	nsresult rv;
 
 	if (mEmbedPersist)
 	{
@@ -103,6 +109,7 @@ MozDownload::Init(nsIURI *aSource, nsILocalFile *aTarget, const PRUnichar *aDisp
 	mPercentComplete = 0;
 	mInterval = 4000; /* in ms */
 	mLastUpdate = mStartTime;
+	mMIMEInfo = aMIMEInfo;
 
 	if (aPersist)
 	{
@@ -259,6 +266,8 @@ NS_IMETHODIMP
 MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 			    PRUint32 aStateFlags, nsresult aStatus)
 {
+	nsresult rv;
+
 	/* For a file download via the external helper app service, we will never get a start
            notification. The helper app service has gotten that notification before it created us. */
 	if (!mGotFirstStateChange)
@@ -299,6 +308,41 @@ MozDownload::OnStateChange (nsIWebProgress *aWebProgress, nsIRequest *aRequest,
 				mozilla_embed_persist_cancelled (mEmbedPersist);
 			}
 		}
+		else if (NS_SUCCEEDED (aStatus))
+		{
+			GnomeVFSMimeApplication *helperApp;
+			char *mimeType;	
+
+			rv = mMIMEInfo->GetMIMEType (&mimeType);
+			NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
+
+			helperApp = gnome_vfs_mime_get_default_application (mimeType); 
+
+			PRUnichar *description;
+			mMIMEInfo->GetApplicationDescription (&description);
+
+			/* HACK we use the application description to decide
+			   if we have to open the saved file */
+			if ((strcmp (NS_ConvertUCS2toUTF8 (description).get(), "gnome-default") == 0) &&
+			    helperApp)
+			{
+				GList *params = NULL;
+				char *param;
+				nsCAutoString aDest;
+
+				mDestination->GetNativePath (aDest);
+
+				param = gnome_vfs_make_uri_canonical (aDest.get ());
+				params = g_list_append (params, param);
+				gnome_vfs_mime_application_launch (helperApp, params);
+				g_free (param);
+
+				g_list_free (params);
+			}
+
+			nsMemory::Free (mimeType);
+			gnome_vfs_mime_application_free (helperApp);
+		}
 	}
         
 	return NS_OK; 
@@ -363,6 +407,12 @@ MozDownload::OnSecurityChange (nsIWebProgress *aWebProgress, nsIRequest *aReques
 void
 MozDownload::Cancel()
 {
+	if (mDownloadState != EPHY_DOWNLOAD_DOWNLOADING &&
+	    mDownloadState != EPHY_DOWNLOAD_PAUSED)
+	{
+		return;
+	}
+	
 	if (mWebPersist)
 	{
 		mWebPersist->CancelSave ();
@@ -410,7 +460,7 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 			 strcmp (contentType, "application/xhtml+xml") == 0));
 
 	nsCOMPtr<nsIWebBrowserPersist> webPersist = do_CreateInstance(persistContractID, &rv);
-	if (NS_FAILED(rv)) return rv;
+	NS_ENSURE_SUCCESS (rv, rv);
   
 	PRInt64 timeNow = PR_Now();
   
@@ -421,7 +471,7 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 	/* dlListener attaches to its progress dialog here, which gains ownership */
 	rv = downloader->InitForEmbed (inOriginalURI, inDestFile, fileDisplayName.get(),
 				       nsnull, timeNow, webPersist, embedPersist);
-	if (NS_FAILED(rv)) return rv;
+	NS_ENSURE_SUCCESS (rv, rv);
 
 	PRInt32 flags = nsIWebBrowserPersist::PERSIST_FLAGS_REPLACE_EXISTING_FILES;
 	if (ephy_flags & EMBED_PERSIST_COPY_PAGE)
@@ -488,39 +538,27 @@ nsresult InitiateMozillaDownload (nsIDOMDocument *domDocument, nsIURI *sourceURI
 static char*
 GetFilePath (const char *filename)
 {
-	char *path, *download_dir;
+	char *path, *download_dir, *expanded, *tmp;
 
 	download_dir = eel_gconf_get_string (CONF_STATE_DOWNLOAD_DIR);
+
 	if (!download_dir)
 	{
 		/* Emergency download destination */
 		download_dir = g_strdup (g_get_home_dir ());
 	}
+	else if (g_utf8_collate (download_dir, "Downloads") == 0)
+	{
+		tmp = g_build_filename (g_get_home_dir (), "Desktop",
+				        _("Downloads"), NULL);
+		g_free (download_dir);
+		download_dir = tmp;
+	}
+
+	expanded = gnome_vfs_expand_initial_tilde (download_dir);
+	path = g_build_filename (expanded, filename, NULL);
+	g_free (expanded);
 
-	if (!strcmp (download_dir, "Desktop"))
-	{
-		if (eel_gconf_get_boolean (CONF_DESKTOP_IS_HOME_DIR))
-		{
-			path = g_build_filename 
-				(g_get_home_dir (),
-				 filename,
-				 NULL);
-		}
-		else
-		{
-			path = g_build_filename 
-				(g_get_home_dir (), "Desktop",
-				 filename,
-				 NULL);
-		}
-	}
-	else
-	{
-		path = g_build_filename
-			(gnome_vfs_expand_initial_tilde (download_dir),
-			 filename,
-			 NULL);
-	}
 	g_free (download_dir);
 
 	return path;
