@@ -1,6 +1,6 @@
 /*
  *  Copyright (C) 2000-2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Christian Persch
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-dialog.c,v 1.23 2003/12/07 10:23:31 chpe Exp $
+ *  $Id: ephy-dialog.c,v 1.26 2004/01/24 09:56:44 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -43,7 +43,8 @@ enum
 {
 	PROP_0,
 	PROP_PARENT_WINDOW,
-	PROP_MODAL
+	PROP_MODAL,
+	PROP_PERSIST_POSITION
 };
 
 typedef enum
@@ -84,6 +85,7 @@ struct EphyDialogPrivate
 	gboolean has_default_size;
 	gboolean disposing;
 	gboolean initialized;
+	gboolean persist_position;
 };
 
 #define SPIN_DELAY 0.20
@@ -968,6 +970,8 @@ setup_default_size (EphyDialog *dialog)
 	{
 		ephy_state_add_window (dialog->priv->dialog,
 				       dialog->priv->name, -1, -1,
+				       dialog->priv->persist_position ?
+				       EPHY_STATE_WINDOW_SAVE_POSITION :
 				       EPHY_STATE_WINDOW_SAVE_SIZE);
 
 		dialog->priv->has_default_size = TRUE;
@@ -989,11 +993,13 @@ static void
 impl_construct (EphyDialog *dialog,
 		const EphyDialogProperty *properties,
 		const char *file,
-		const char *name)
+		const char *name,
+		const char *domain)
 {
 	GladeXML *gxml;
 
-	gxml = ephy_glade_widget_new (file, name, &(dialog->priv->dialog), dialog);
+	gxml = ephy_glade_widget_new
+		(file, name, &(dialog->priv->dialog), dialog, domain);
 
 	if (dialog->priv->name == NULL)
 	{
@@ -1139,10 +1145,11 @@ void
 ephy_dialog_construct (EphyDialog *dialog,
 		       const EphyDialogProperty *properties,
 		       const char *file,
-		       const char *name)
+		       const char *name,
+		       const char *domain)
 {
 	EphyDialogClass *klass = EPHY_DIALOG_GET_CLASS (dialog);
-	return klass->construct (dialog, properties, file, name);
+	return klass->construct (dialog, properties, file, name, domain);
 }
 
 void
@@ -1152,6 +1159,15 @@ ephy_dialog_show (EphyDialog *dialog)
 	klass->show (dialog);
 }
 
+void
+ephy_dialog_hide (EphyDialog *dialog)
+{
+	g_return_if_fail (EPHY_IS_DIALOG (dialog));
+	g_return_if_fail (dialog->priv->dialog != NULL);
+
+	gtk_widget_hide (dialog->priv->dialog);
+}
+
 #if 0
 static void
 run_response_cb (GtkWidget *dialog,
@@ -1262,6 +1278,7 @@ ephy_dialog_init (EphyDialog *dialog)
 	dialog->priv->initialized = FALSE;
 	dialog->priv->has_default_size = FALSE;
 	dialog->priv->disposing = FALSE;
+	dialog->priv->persist_position = FALSE;
 
 	dialog->priv->props = g_hash_table_new_full 
 		(g_str_hash, g_str_equal, NULL, (GDestroyNotify) free_prop_info);
@@ -1317,6 +1334,9 @@ ephy_dialog_set_property (GObject *object,
 		case PROP_MODAL:
 			ephy_dialog_set_modal (dialog, g_value_get_boolean (value));
 			break;
+		case PROP_PERSIST_POSITION:
+			dialog->priv->persist_position = g_value_get_boolean (value);
+			break;
 	}
 }
 
@@ -1335,6 +1355,10 @@ ephy_dialog_get_property (GObject *object,
 			break;
 		case PROP_MODAL:
 			g_value_set_boolean (value, dialog->priv->modal);
+			break;
+		case PROP_PERSIST_POSITION:
+			g_value_set_boolean (value, dialog->priv->persist_position);
+			break;
 	}
 }
 
@@ -1380,6 +1404,14 @@ ephy_dialog_class_init (EphyDialogClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	g_object_class_install_property (object_class,
+					 PROP_PERSIST_POSITION,
+					 g_param_spec_boolean ("persist-position",
+							       "Persist position",
+							       "Persist dialog position",
+							       FALSE,
+							       G_PARAM_READWRITE));
+
 	g_type_class_add_private (object_class, sizeof (EphyDialogPrivate));
 }
 
